/*
 ** colordialog.cpp
 **	by Saki Sakira <sakira@sun.dhis.portside.net>
 **	from 2003 June 8
 */

#include <qapplication.h>
#include <stdio.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qslider.h>
#include <qlabel.h>
#include <qstring.h>
#include <qgrid.h>

#include "colordialog.h"

////////// ColorDialog

ColorDialog::ColorDialog(QColor &col,
                         const char* name)
     : QDialog(0, name, TRUE)
{
  setCaption(name);

  int h, s, v;
  init_col = new QColor(col);
  cur_col = &col;
  col.hsv(&h, &s, &v);
  QString nstr("");
  QString ok_str("OK");
  QString cancel_str("Cancel");
  
  box = new QVBox(this);

  if (qApp->desktop()->height() <= 240) {
    setFont(QFont("lcfont", 8));
    box->setMinimumSize(240, 220);
  } else {
    box->setMinimumSize(240, 300);
  }

  hue_sat = new HueSat(box);
  hue_sat->set_cursor(*init_col);
  
  val_box = new QHBox(box);
  val_box->setSpacing(2);
  val = new ValLine(val_box);
  val->set_cursor(*cur_col);
  
  sliders = new QGrid(2, box);
  sliders->setSpacing(0);
  red_num = new QLabel("R: 000", sliders);
  red_slider = new QSlider(0, 255, 16, col.red(),
                       QSlider::Horizontal, sliders);
  green_num = new QLabel("G: 000", sliders);
  green_slider = new QSlider(0, 255, 16, col.green(),
                       QSlider::Horizontal, sliders);
  blue_num = new QLabel("B: 000", sliders);
  blue_slider = new QSlider(0, 255, 16, col.blue(),
                       QSlider::Horizontal, sliders);

  if (qApp->desktop()->height() <= 240) {
    red_slider->setMaximumHeight(10);
    green_slider->setMaximumHeight(10);
    blue_slider->setMaximumHeight(10);
  }
  
  col_boxes = new QHBox(box);
  col_boxes->setSpacing(2);
  init_cb = new ColorBox(col_boxes);
  init_cb->setColor(*init_col);
  cur_cb = new ColorBox(col_boxes);
  cur_cb->setColor(*cur_col);

  connect(hue_sat, SIGNAL(valueChanged(int, int)),
          val, SLOT(setHueSat(int, int)));
  connect(val, SIGNAL(valueChanged(QColor)),
          cur_cb, SLOT(setColor(QColor)));

  connect(red_slider, SIGNAL(valueChanged(int)),
          this, SLOT(getSliders(int)));
  connect(green_slider, SIGNAL(valueChanged(int)),
          this, SLOT(getSliders(int)));
  connect(blue_slider, SIGNAL(valueChanged(int)),
          this, SLOT(getSliders(int)));
  connect(red_slider, SIGNAL(sliderReleased(void)),
          this, SLOT(fixSliders(void)));
  connect(green_slider, SIGNAL(sliderReleased(void)),
          this, SLOT(fixSliders(void)));
  connect(blue_slider, SIGNAL(sliderReleased(void)),
          this, SLOT(fixSliders(void)));
  
  connect(init_cb, SIGNAL(clicked(QColor)),
          hue_sat, SLOT(set_cursor(QColor)));
  connect(init_cb, SIGNAL(clicked(QColor)),
          val, SLOT(set_cursor(QColor)));
  connect(init_cb, SIGNAL(clicked(QColor)),
          cur_cb, SLOT(setColor(QColor)));

  connect(cur_cb, SIGNAL(valueChanged(QColor)),
          this, SLOT(set_current_color(QColor)));

  set_current_color(col);
}

ColorDialog::~ColorDialog() {
  hue_sat->erase_cursor();
  
  delete init_cb;
  delete cur_cb;
  delete col_boxes;

  delete val;
  delete val_box;

  delete hue_sat;

  delete red_num;
  delete green_num;
  delete blue_num;
  delete red_slider;
  delete green_slider;
  delete blue_slider;
  delete sliders;
  
  delete box;
}

void ColorDialog::getSliders(int) {
  int r, g, b;

  r = red_slider->value();
  g = green_slider->value();
  b = blue_slider->value();

  QColor col(r, g, b);
  
  set_current_color(col);
}

void ColorDialog::fixSliders(void) {
  getSliders(0);

  hue_sat->set_cursor(*cur_col);
  val->set_cursor(*cur_col);
  cur_cb->setColor(*cur_col);
}

void ColorDialog::set_current_color(QColor col) {
  int r, g, b;

  r = col.red();
  g = col.green();
  b = col.blue();

  cur_col->setRgb(r, g, b);

  red_num->setText("R: " + QString::number(r));
  green_num->setText("G: " + QString::number(g));
  blue_num->setText("B: " + QString::number(b));
  
  red_slider->setValue(r);
  green_slider->setValue(g);
  blue_slider->setValue(b);
}

////////// HueSat


QPixmap *HueSat::pix = 0;
QPoint HueSat::cursor_pt(-1, -1);

HueSat::HueSat(QWidget *w) : QWidget(w) {
  setFixedSize(HueCount, SaturationCount);

  setMouseTracking(FALSE);

  init_pix();
}

void HueSat::init_pix(void) {
  if (pix) return;
  
  pix = new QPixmap(HueCount, SaturationCount);

  QPainter p(pix);
  QColor col;
  int x, y, h, s;

  for (x = 0; x < 240; x ++) {
    h = (int)(((float)x*360)/HueCount);
    for (y = 0; y < 120; y ++) {
      s = (int)(((float)y*256)/SaturationCount);
      col.setHsv(h, s, 255);
      p.setPen(col);
      p.drawPoint(x, y);
    }
  }
}

void HueSat::paintEvent(QPaintEvent *) {
  bitBlt(this, 0, 0, pix, 0, 0, width(), height(),
         CopyROP, FALSE);
}

void HueSat::erase_cursor(void) {
  const int r = 5;
  QPainter pf(this);
  QPainter pb(pix);
  QPen pen(white, 2);

  pf.setPen(pen);
  pb.setPen(pen);
  pf.setRasterOp(XorROP);
  pb.setRasterOp(XorROP);

  if (cursor_pt.x() >= 0) {
    pf.drawEllipse(cursor_pt.x() - r/2,
                   cursor_pt.y() - r/2, r, r);
    pb.drawEllipse(cursor_pt.x() - r/2,
                   cursor_pt.y() - r/2, r, r);
  }

  cursor_pt.setX(-1);
  cursor_pt.setY(-1);
}

void HueSat::set_cursor(int x, int y) {
  const int r = 5;
  QPainter pf(this);
  QPainter pb(pix);
  QPen pen(white, 2);

  if (x < 0) x = 0;
  if (x >= HueCount) x = HueCount - 1;
  if (y < 0) y = 0;
  if (y >= SaturationCount) y = SaturationCount - 1;

  erase_cursor();
  
  pf.setPen(pen);
  pb.setPen(pen);
  pf.setRasterOp(XorROP);
  pb.setRasterOp(XorROP);

  pf.drawEllipse(x - r/2, y - r/2, r, r);
  pb.drawEllipse(x - r/2, y - r/2, r, r);
  
  cursor_pt.setX(x);
  cursor_pt.setY(y);

  int h, s;
  h = (int)(((float)x*360)/HueCount);
  s = (int)(((float)y*256)/SaturationCount);
  emit valueChanged(h, s);
}

void HueSat::set_cursor(QColor c) {
  int h, s, v, x, y;

  c.hsv(&h, &s, &v);
  x = (int)(((float)h*HueCount)/360);
  y = (int)(((float)s*SaturationCount)/256);

  set_cursor(x, y);
}

void HueSat::mousePressEvent(QMouseEvent *e) {
  set_cursor(e->x(), e->y());
}

void HueSat::mouseMoveEvent(QMouseEvent *e) {
  mousePressEvent(e);
}


////////// ValLine

ValLine::ValLine(QWidget *p, int w, int h)
     : QWidget(p)
{
  setFixedSize(w, h);
  
  pix = 0;
}

void ValLine::paintEvent(QPaintEvent *) {
  if (!pix) set_pix(0, 0);

  bitBlt(this, 0, 0, pix, 0, 0, width(), height(),
         CopyROP, FALSE);

}

void ValLine::set_pix(QColor col) {
  int h, s, v;

  col.hsv(&h, &s, &v);
  set_pix(h, s);
}

void ValLine::set_pix(int h, int s) {
  pix = new QPixmap(size());

  QPainter p(pix);
  QColor col;
  int x, v, w, ly;
  
  w = width();
  ly = height() - 1;
  for (x = 0; x < w; x ++) {
    v = (int)(((float)x*256)/w);
    col.setHsv(h, s, v);
    p.setPen(col);
    p.drawLine(x, 0, x, ly);
  }

  hue = h;
  saturation = s;
  cursor_x = -1;
}

void ValLine::setHueSat(int h, int s) {
  set_pix(h, s);
  repaint();
}

void ValLine::set_cursor(QColor c) {
  int h, s, v, x;

  c.hsv(&h, &s, &v);
  x = (int)(((float)v*width())/256);
  set_pix(h, s);
  set_cursor(x);
  
  repaint();
}

void ValLine::set_cursor(int x) {
  QPainter p(pix);
  QPen pen(white, 2);

  if (x < 0) x = 0;
  if (x >= width()) x = width() - 1;

  p.setPen(pen);
  p.setRasterOp(XorROP);

  if (cursor_x >= 0)
    p.drawLine(cursor_x, height()/2, cursor_x, height() - 1);

  p.drawLine(x, height()/2, x, height() - 1);
  repaint();

  cursor_x = x;

  int value = (int)(((float)x*256)/width());
  QColor col;
  col.setHsv(hue, saturation, value);
  emit valueChanged(col);
}

void ValLine::mousePressEvent(QMouseEvent *e) {
  set_cursor(e->x());
}

void ValLine::mouseMoveEvent(QMouseEvent *e) {
  mousePressEvent(e);
}


////////// ColorBox

ColorBox::ColorBox(QWidget *w) : QWidget(w) {
  parent = w;
  
  setFixedSize(50, 20);
  color = QColor(white);
}

void ColorBox::setColor(QColor col) {
  if (color == col) return;
  
  color = col;
  repaint();

  emit valueChanged(col);
}

void ColorBox::paintEvent(QPaintEvent *) {
  QPainter p(this);
  QBrush b(color);

  p.setBrush(b);
  p.drawRect(0, 0, width() - 1, height() - 1);
}

void ColorBox::mousePressEvent(QMouseEvent *) {
  emit clicked(color);
}
