#ifndef COLORDIALOG_H
#define COLORDIALOG_H

#include <qdialog.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qpushbutton.h>

#define HueCount 240
#define SaturationCount 120

class QSlider;
class QLabel;
class QGrid;

class HueSat;
class ValLine;
class ColorBox;

class ColorDialog : public QDialog {
  Q_OBJECT

public:
  ColorDialog(QColor &init_col,
              const char* name = 0);
  ~ColorDialog();

public slots:
  void getSliders(int);
  void fixSliders(void);
  void set_current_color(QColor);

private:
  QVBox *box;
  HueSat *hue_sat;
  ValLine *val;
  ColorBox *init_cb, *cur_cb;
  QHBox *col_boxes, *val_box;
//  QHBox *col_boxes, *buttons, *val_box;
//  QPushButton *ok_btn, *cancel_btn;
  QColor *init_col, *cur_col;
  QGrid *sliders;
  QLabel *red_num, *green_num, *blue_num;
  QSlider *red_slider, *green_slider, *blue_slider;
};

class HueSat : public QWidget {
  Q_OBJECT

public:
  HueSat(QWidget *);

  void paintEvent(QPaintEvent *);
  void erase_cursor(void);
  void set_cursor(int x, int y);
  void mousePressEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);

public slots:
  void set_cursor(QColor);
  
signals:
  void valueChanged(int, int);
  
private:
  static QPixmap *pix;
  static QPoint cursor_pt;

  static void init_pix(void);

};

class ValLine : public QWidget {
  Q_OBJECT

public:
  ValLine(QWidget *, int = 128, int = 20);
  ~ValLine() {delete pix;};

  void paintEvent(QPaintEvent *);
  void set_pix(QColor col);
  void set_pix(int = 0, int = 0);
  void mousePressEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);
  void set_cursor(int);

public slots:
  void setHueSat(int, int);
  void set_cursor(QColor);

signals:
  void valueChanged(QColor);
  
private:
  int cursor_x;
  int hue, saturation;
  QPixmap *pix;
};

class ColorBox : public QWidget {
  Q_OBJECT

public:
  ColorBox(QWidget *w = 0);

  void paintEvent(QPaintEvent *);
  void mousePressEvent(QMouseEvent *);

public slots:
  void setColor(QColor);

signals:
  void clicked(QColor);
  void valueChanged(QColor);
  
private:
  QColor color;
  QWidget *parent;

};

#endif //	COLORDIALOG_H
