/*
 **	colorpalette.cpp
 **	by Saki Sakira <sakira@sun.dhis.portside.net>
 **	from 2003 September 23
 */

#include <qsize.h>
#include <qcolor.h>
#include <qarray.h>
#include <qfile.h>
#include <qimage.h>
#include <qpe/resource.h>

#include "settings.h"
#include "sketchpainter.h"
#include "colorpalette.h"

const QString ColorPalette::Filename = "petitpeintu_colorpallete.png";
QArray<uint16> ColorPalette::colors(ColorPalette::length);
const uint16 ColorPalette::init_colors[] =
{0x0000, 0x1000, 0x0100, 0x0010, 0x0001,
  0x8000, 0x0800, 0x0080, 0x0008, 0xffff};
QArray<bool> ColorPalette::changeds;
QImage ColorPalette::img;
Settings* ColorPalette::settings = 0;

ColorPalette::ColorPalette(int w, int h) {
  width = w;
  height = h;

  if (changeds.size() == 0) {
    changeds.fill(false, length);
    colors.resize(length);
  }

  if (settings)
    settings->getPaletteColors(colors, changeds);

}

ColorPalette::~ColorPalette(void) {
  settings->save();
}

void ColorPalette::setSettings(Settings *s) {
  settings = s;
}

void ColorPalette::setImage(const QString &fn2) {
  QString fn = fn2;

  if (fn == "")
    fn = settings->getPaletteFileName();

  if (!QFile::exists(fn))
    fn = Resource::findPixmap(ColorPalette::Filename);
  if (!QFile::exists(fn))
    qFatal("palette image %s does not exist\n", fn.latin1());

  img.load(fn);
  img.convertDepth(32);
}

void ColorPalette::setColors(const uint16* cs) {
  for (uint i = 0; i < length; i ++)
    colors[i] = cs[i];
}

void ColorPalette::setChangeds(const bool *cs) {
  if (cs)
    for (uint i = 0; i < length; i ++)
      changeds[i] = cs[i];
  else
    changeds.fill(false, length);
}

void ColorPalette::setChanged(uint i, bool f) {
  changeds[i] = f;

  if (settings)
    settings->setPaletteColors(colors, changeds);

}

uint ColorPalette::getIndex(uint x, uint) {
  return x * length / width;
}

QSize ColorPalette::size(void) {
  return QSize(width, height);
}

QColor ColorPalette::getColor(uint x, uint) {
  return SketchPainter::unpack_color(colors[getIndex(x)]);
}

QArray<uint16> ColorPalette::getColors(void) {
  QArray<uint16> rcolors(length);

  for (uint i = 0; i < length; i ++)
    rcolors[i] = colors[i];

  return rcolors;
}

void ColorPalette::infoImage(uint16 *buf) {

  if (img.width() > 0) {
    QColor col;
    uint x, y;

    for (y = 0; y < height; y ++)
      for (x = 0; x < width; x ++) {
        col.setRgb(img.pixel(img.width() * x / width,
                             img.height() * y / height));
        buf[y * width +  x] =
          SketchPainter::pack_color(col);
      }
  } else {
    for (uint i = 0; i < length; i ++)
      if (!changeds[i]) {
        colors[i] = init_colors[i];
        changeds[i] = true;
      }
  }

  uint x, y, index;
  uint16 c;
  int i;

  for (x = 0; x < width; x ++) {
    i = x * length / width;
    if (changeds[i]) {
      c = colors[i];
      index = x;
      for (y = 0; y < height; y ++) {
        buf[index] = c;
        index += width;
      }
    }
  }
}

void ColorPalette::setColor(uint i, uint16 c) {
  colors[i] = c;
  changeds[i] = true;

  if (settings)
    settings->setPaletteColors(colors, changeds);
}

void ColorPalette::setColor(uint x, uint, uint16 c) {
  setColor(x * length / width, c);
}
