#ifndef COLORPANEL_H
#define COLORPANEL_H

#include <qobject.h>
#include <qrect.h>

#include "constants.h"
#include "rgb_hsv.h"

class QSize;
class QColor;

class ColorPalette;
class HueLine;
class SatLum;

class ColorPanel : public QObject {
  Q_OBJECT

  uint16 *buf;
  uint width, height;
  RGB_HSV rgb_hsv;
  int sat_num, lum_num;
  HueLine *hue;
  SatLum *sat_lum;
  QRect dlg_btn, cancel_btn, ok_btn;
  QRect palette_rect, hue_rect, satlum_rect;
  bool user_palette;
  ColorPalette *palette;
//  bool palette_changing;
  int mode;
  enum {ModeNone = 0,
    ModePaletteSetting, ModePaletteClearing,
    ModeSatLum,
    NumOfModes};
  uint palette_index;

  void fillRect(const QRect&, uint16);

public:
  ColorPanel(int = 240, int = 240,
             const QColor &init_col = Qt::red);
  ~ColorPanel(void);

  QSize size(void);
  QColor getColor(uint, uint);

  uint16* infoImage(void);
  void setUserPalette(const QString & = "");
  QPoint huePoint(int);
  QPoint satlumPoint(int);

public slots:
  void setColor(const QColor &);
  void setInitColor(const QColor &);
  void setHue(uint);
  void clicked(int x, int y, bool = true);
  void released(int, int);
  void setHueSatLum(int, int, int);

signals:
  void selected(uint16);
  void finished(void);
  void showDialog(QColor);
  void paletteChanged(void);
};

class HueLine : public QObject {
  Q_OBJECT

  int width, height;
  uint16 *buf;

public:
  HueLine(int, int);
  ~HueLine(void) {
    delete buf;
  }

public slots:
  void clicked(uint);
  
signals:
  void hueChanged(uint);
  
protected:
  friend class ColorPanel;
};

class SatLum : public QObject {
  Q_OBJECT

  uint16 *buf;
  uint hue;

  static const int width = 171;
  static const int height = 171;

public:
  SatLum(uint hue = 0);
  ~SatLum(void) {
    delete buf;
  }
  
public slots:
  void setColor(uint hue);
  void clicked(int x, int y);

signals:
  void selected(int, int, int);
  
protected:
  friend class ColorPanel;
};
      
#endif		// COLORPANEL_H
