/*!
 * filedialog.cpp
 * Original
 * by Saki Sakira <sakira@sun.dhis.portside.net>
 * from 2003 June 18
 *
 * Customize(perhaps,"Renew")
 * by Reini Kasumidoki <di-azz@mc.kcom.ne.jp>
 * from 2003 July 09
 *
 * Modify(QListBox -> QIconView)
 * by Tomokazu Sato <s-tomo@iijmio-mail.jp>
 * 2003 Nov 19
 */

#include <stdio.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qpe/resource.h>

#include "ptptformat.h"
#include "filedialog.h"


//! fileinfo is static member.
QFileInfo FileDialog::fileinfo(QDir::homeDirPath());

//! construct
FileDialog::FileDialog(const char *name, bool input,const QString &path)
     : QDialog(0, name, TRUE)
{

  //QDialog
  setCaption(name);
  //setSizeGripEnabled(true);
  
  //QLayout
  QLayout *layout = new QVBoxLayout(this);
  layout->setAutoAdd(true);

  //QLineEdit -Input-
  fdir = new QLineEdit(this);
  fdir->setReadOnly(TRUE);
  //fdir->setFixedHeight(fdir->height());

  //QFileInfo
  if (!path.isNull()){
    QFileInfo info(path);
    QDir infodir = info.dir();
    while (!infodir.exists()) infodir.cdUp();
    info.setFile(infodir, info.fileName());
    fileinfo = info;
    fileinfo.convertToAbs();
  }
  
  //QIconView
  flist = new QIconView(this);
  set_flist();
  flist->setFocus();                //GetFoucus
  flist->installEventFilter(this);  //EventFilter
  connect(flist, SIGNAL(doubleClicked(QIconViewItem *)),
	  this,  SLOT(select_file(QIconViewItem *)));
  connect(flist, SIGNAL(returnPressed(QIconViewItem *)),
	  this,  SLOT(select_file(QIconViewItem *)));

  //static fileinfo
  QString strName;
  if(fileinfo.isDir() == false){
    strName = fileinfo.fileName();
    
    fileinfo.setFile(QDir::cleanDirPath(fileinfo.dirPath(true)));
    fileinfo.convertToAbs();
  }

  //QLineEdit -Output-
  if (!input){
    fedit = new QLineEdit(strName, this);
    if (fedit) {
      connect(fedit, SIGNAL(returnPressed()), this, SLOT(set_editEnd()));
      //fedit->setFixedHeight(fedit->height());
    }
  }else{
    fedit = NULL;
  }

  showMaximized();
}

//! destruct
FileDialog::~FileDialog() {
  delete fedit;
  delete flist;
  delete fdir;
}

//	modified by Saki Sakira (2003.07.10)
//! fileinfo -> flist(exists & dir only)
void FileDialog::set_flist(){
  //fileinfo check
  //  if(fileinfo.exists() == false){return;}
  QFileInfo info = fileinfo;
  while (!info.exists())
    info.setFile(info.dir().path());
  
  //list
  flist->clear();
  QDir dir((fileinfo.isDir()) ? fileinfo.filePath() : fileinfo.dir());
  const int iFilter = QDir::DefaultFilter;//QDir::Dirs | QDir::Files | QDir::Readable | QDir::Writable;
  const int iSort   = QDir::Name;

  QFileInfoListIterator it(*dir.entryInfoList(iFilter, iSort));
  for(uint i = 0; i < it.count(); i++, ++it) {
    QFileInfo *info = it.current();
    if(info->isDir()){
      new QIconViewItem(flist, info->fileName(), Resource::loadPixmap("slfolder_l"));
      QString fn = Resource::findPixmap("slfolder_l");
      if (!QFile::exists(fn))	// for non-zaurus, e.g. Opie
        fn = Resource::findPixmap("folder");
      new QIconViewItem(flist, info->fileName(), QPixmap(fn));
    }else{
      QString thumb = info->baseName() + "_" + info->extension() + ".jpg";
      QPixmap pixmap(info->dirPath() + "/" +
                     PtptFormat::dotIcon() + "/" + thumb);
      new QIconViewItem(flist, info->fileName(), pixmap);
    }
  }

  //path
  fdir->setText(dir.absPath());
}

//! get a fullpath
QString FileDialog::filePath(void) {return fileinfo.filePath();}

//! signal event[list selected]
void FileDialog::select_file(const QString & str) {
  {
    QFileInfo info((fileinfo.isDir()) ? fileinfo.filePath() : fileinfo.dir(),str);
    if(info.exists() == true){
      fileinfo = info;
      fileinfo.convertToAbs();
    }else{
      return;
    }
  }

  if (fileinfo.isDir()) {
    fileinfo.convertToAbs();
    set_flist();
  } else {
    if (fedit){
      fedit->setText(str);
      set_editEnd();
    }else{
      accept();
    }
  }
}

//	modified by Saki Sakira (2003.07.10)
//! signal event[OK]
void FileDialog::accept(){
  if(fileinfo.isDir() == true){
    if (fedit && !fedit->text().isNull()) {
      fileinfo.setFile((fileinfo.isDir()) ? fileinfo.filePath() : fileinfo.dir(),fedit->text());
      fileinfo.convertToAbs();
      fedit->setText(QString::null);

      accept();
    } else {
      select_file(flist->currentItem()->text());
    }
  }else{
    QDialog::accept();
  }
}

//! signal event[edit hitRet] fedit -> fileinfo ->accept
void FileDialog::set_editEnd(){
  fileinfo.setFile((fileinfo.isDir()) ? fileinfo.filePath() : fileinfo.dir(),fedit->text());
  fileinfo.convertToAbs();
  accept();
}

//! hook keyEvent[list hitBackSpace]
bool FileDialog::eventFilter( QObject *pObj, QEvent *pEve ){
  if(pObj == flist){
    if( pEve->type() == QEvent::KeyPress ){
      QKeyEvent *pKeyEve = (QKeyEvent*)pEve;
      switch(pKeyEve->key()){
      case Key_Backspace://upper directory
        select_file("..");
        return true;
        break;
      }
    }
  }
  return QWidget::eventFilter( pObj, pEve );// standard event processing
}

void FileDialog::select_file( QIconViewItem *item )
{
  select_file(item->text());
}
