#ifndef LAYEREDPAINTER_H
#define LAYEREDPAINTER_H

#include "sketchpainter.h"
#include "constants.h"
#include "clipboard.h"
#include "ptpen.h"

class ToolPanel;

class LayeredPainter : public SketchPainter {
  Q_OBJECT
    
  QArray<SketchPainter*> layers;
  SketchPainter *temp_layer;
  static QBitArray *alpha_mask;
  uint cur;
  ToolPanel *tools;
  QRect info_rect, tools_rect;
  bool showing_info, showing_tools;
  QArray<int> sqrt_tbl;

public:
  enum {BaseLayer = 0, ColorLayer, OverLayer,
    NumOfLayers};
  static const int short_shift = 8;

  LayeredPainter(int w = 0, int h = 0);
//  LayeredPainter(const QSize &);
  ~LayeredPainter();

  void setSize(int, int, int = 0);
  void setOrientation(int);
  void setPen(const PtPen &, bool tool = true, bool = false);
  PtPen getPen(void);
  void setPenDensity(int, bool tool = true, bool = false);
  uint getPenDensity(void);
  void setPenMethod(int);
  int getPenMethod(void);
  void clearMask(void);
  
  void setToolPanelActive(int, bool);
  void setToolPanel(ToolPanel *t);

  QColor paperColor(void) {return layers[cur]->paperColor();};
  void setPaperColor(QColor c);
  void clear(void);
  void fill(const QColor &c = Qt::white, bool = true);
  void setLayer(int = -1);
  void setLayerAlpha(int);
  void setLayerAlpha(int, int);
  void switchLayer(int = 1);
  void setCompositionMethod(int);
  void setCompositionMethod(int, int);
  void updateRect(const QRect &r);
  int numOfLayers(void) {return layers.size();}
  uint current(void) {return cur;}

  void joinShowingLayers(void);
  void backupCurrentLayer(void);
  void restoreCurrentLayer(void);
  void exchangeCurrentLayer();
  bool getShowing(void);
  void setShowing(bool);
  bool getShowing(int);
  void setShowing(int, bool);

  void createLayer(void);
  void duplicateCurrentLayer(void);
  void deleteCurrentLayer(void);
  
  void exchangeLayers(int, int);

  void drawPoint(int, int);
  void drawLine(int, int, int, int, bool = false);
  void drawPolyline(const QPointArray &pta);

  QRect& showPenInfo(void);
  QRect& hideInfo(void);

  void infoStrings(QStringList &);

  void mirrorHorizontal(void);
  void mirrorVertical(void);
  void rotateCW(void);
  void rotateCCW(void);

  QPoint shiftCurrentLayer(QPoint &, QRect &);
  QPoint shiftCurrentLayer(QPoint &);
  void shiftAllLayers(QPoint &);
  void unshiftCurrentLayer(void);
  void fixShiftCurrentLayer(void);
  void scaleCurrentLayer(uint, uint);

  void copyClipboard(QRect &, uint);
  void pasteClipboard(QPoint &, uint);
  void copyClipboard(QRect &rect) {copyClipboard(rect, cur);}
  void pasteClipboard(QPoint &pt) {pasteClipboard(pt, cur);}
  bool clipboardPasteMode(void) {return clipboard.pasteMode();}
  void setClipboardPasteMode(bool pm) {clipboard.setPasteMode(pm);}
  QRect clipboardRect(void) {return clipboard.rect();}
  PPClipboard* getClipboard(void) {return &clipboard;}

  void setShowingTools(bool);
//  QRect& updateToolPanel(void);

  bool load(const QString &, const char* = 0);
  bool loadPtpt(const QString &);
  bool loadLayer(const QString &);
  bool save(const QString &, const char*);
  bool savePtpt(const QString &);
  bool saveLayer(const QString &);
  static bool fileExists(const QString &);

protected:
  void setup_sqrt_tbl(void);
  void prepare_layer(SketchPainter *);
  void prepare_current(void);
  //  inline uint16 update_over16(uint16, uint16, uint16, uint);
  inline uint16 update_over16(uint16 c0, uint16 c1,
                              uint16 paper_col, uint alpha) {
    if (c1 == paper_col)
      return c0;
    else
      return tighten((disperse(c0) * (32 - alpha)
                      + disperse(c1) * alpha) >> 5);
  }
  inline uint32 update_over32(uint32, uint32, uint16, uint);
  void update_rect_copy(uint32*, const QRect &);
  void update_rect_min(uint32*, uint32*, const QRect &);
  void update_rect_max(uint32*, uint32*, const QRect &);
  void update_rect_mul(uint32*, uint32*, const QRect &);
  void update_rect_screen(uint32*, uint32*, const QRect &);
  void update_rect_sat(uint16*, uint16*, const QRect &);
  void update_rect_col(uint16*, uint16*, const QRect &);
  void update_rect_normal(SketchPainter*, SketchPainter*,
                          const QRect &);
  void update_rect_mask(uint16*, uint16*, uint16*, const QRect &);
  void update_rect_clipboard(void);
  void copy_entire_buf(SketchPainter&, SketchPainter&);
  void copy_entire_buf_with_scaling(
    SketchPainter&, SketchPainter&, uint, uint);
  void exchange_entire_buf(SketchPainter&, SketchPainter&);
  void draw_line_with_alpha(int, int, int, int,
                            SketchPainter*, SketchPainter*);

#include "painter.inline"
  
signals:
  void toolPanelChanged(void);
  void showingChanged(int, bool);
  void compositionChanged(int, int);
  void alphaChanged(int, int);
  void currentChanged(int);
  void numOfLayersChanged(int);
  
};

#endif //	 LAYEREDPAINTER_H
