/*
 **	layerpanel.cpp
 **	written by Saki Sakira <sakira@sun.dhis.portside.net>
 **	from 2003 September 4
 */

#include <qpainter.h>

#include "constants.h"
#include "layerpanel.h"

LayerPanel::LayerPanel(int n) {
  top_left_point.setX(0);
  top_left_point.setY(0);

  show = false;

  setNumOfLayers(n);
}

void LayerPanel::setNumOfLayers(int n) {
  num_of_layers = n;
  
  showing_rects.resize(n);
  composition_rects.resize(n);
  alpha_rects.resize(n);
  tab_rects.resize(n);

  showings.resize(n);
  compositions.resize(n);
  alphas.resize(n);

  if (n >= 1) {
    pix.resize(240, 240);

    QRect prect = pix.rect();
    QPainter p(&pix);
    QRect srect =
      p.boundingRect(prect, Qt::AlignLeft, " S");
    prect.moveBy(srect.width(), 0);
    QRect crect =
      p.boundingRect(prect, Qt::AlignLeft, " C");
    prect.moveBy(crect.width(), 0);
    QRect arect =
      p.boundingRect(prect, Qt::AlignLeft, "000");
    prect.moveBy(arect.width(), 0);
    QRect trect =
      p.boundingRect(prect, Qt::AlignLeft, "@");
    p.end();

    int h = srect.height();
    pix.resize(srect.width() + crect.width() +
               arect.width() + trect.width(),
               h * n);
    for (int i = 0; i < n; i ++) {
      showing_rects[n - 1 - i] = srect;
      composition_rects[n - 1 - i] = crect;
      alpha_rects[n - 1 - i] = arect;
      tab_rects[n - 1 - i] = trect;
      srect.moveBy(0, h);
      crect.moveBy(0, h);
      arect.moveBy(0, h);
      trect.moveBy(0, h);
    }
  }
}

bool LayerPanel::showing(int i) {
  return showings[i];
}

void LayerPanel::setShowing(int i, bool f) {
  showings[i] = f;
}

void LayerPanel::setShow(bool f) {
  show = f;
}

int LayerPanel::composition(int i) {
  return compositions[i];
}

void LayerPanel::setComposition(int l, int c) {
  compositions[l] = c;
}

int LayerPanel::alpha(int i) {
  return alphas[i];
}

void LayerPanel::setAlpha(int i, int a) {
  alphas[i] = a;
}

void LayerPanel::setTopLeft(QPoint &pt) {
  top_left_point = pt;
}

QRect LayerPanel::rect(void) {
  if (show)
    return QRect(top_left_point, pix.size());
  else
    return QRect(0, 0, 0, 0);
}

QSize LayerPanel::size(void) {
  return pix.size();
}

QString& LayerPanel::compositionString(int c) {
  static QString cstr;

  switch (c) {
  case MinComposition:
    cstr = "_";
    break;
  case MulComposition:
    cstr = "*";
    break;
  case ScreenComposition:
    cstr = "/";
    break;
  case SatComposition:
    cstr = "S";
    break;
  case ColComposition:
    cstr = "C";
    break;
  case NormalComposition:
    cstr = "N";
    break;
  case MaxComposition:
    cstr = "^";
    break;
  case MaskComposition:
    cstr = "M";
    break;
  case AlphaChannelComposition:
    cstr = "A";
    break;
  }

  return cstr;
}

QPixmap* LayerPanel::pixmap(void) {
  pix.fill();

  QPainter p(&pix);
  p.setPen(Qt::black);
  for (uint i = 0; i < num_of_layers; i ++) {
    if (showings[i])
      p.drawText(showing_rects[i], Qt::AlignCenter, "S");
    else
      p.drawText(showing_rects[i], Qt::AlignCenter, "H");

    p.drawText(composition_rects[i], Qt::AlignCenter,
               compositionString(compositions[i]));
    p.drawText(alpha_rects[i], Qt::AlignRight,
               QString::number(alphas[i]));
    p.drawText(tab_rects[i], Qt::AlignCenter, "@");
  }
  if (current >= 0 && current < (int)num_of_layers) {
    p.setRasterOp(Qt::XorROP);
    p.fillRect(showing_rects[current], Qt::white);
    p.fillRect(composition_rects[current], Qt::white);
    p.fillRect(alpha_rects[current], Qt::white);
    p.fillRect(tab_rects[current], Qt::white);
  }
  p.end();

  return &pix;
}

uint LayerPanel::layerNum(int y) {
  for (int i = num_of_layers - 1; i >= 0; i --)
    if (y <= showing_rects[i].bottom())
      return i;

  return num_of_layers;
}

uint LayerPanel::typeNum(int x) {
  if (x <= showing_rects[0].right())
    return Show;
  else if (x <= composition_rects[0].right())
    return Composition;
  else if (x <= alpha_rects[0].right())
    return Alpha;
  else
    return Tab;
}
