#ifndef LAYERPANEL_H
#define LAYERPANEL_H

#include <qpixmap.h>
#include <qpoint.h>
#include <qarray.h>
#include <qobject.h>

class SketchPainter;

class LayerPanel : public QObject {
  Q_OBJECT
  
  QPixmap pix;
  uint num_of_layers;
  int current;
  bool show;
  QPoint top_left_point;

  QArray<QRect> showing_rects;
  QArray<QRect> composition_rects;
  QArray<QRect> alpha_rects;
  QArray<QRect> tab_rects;

  QArray<bool> showings;
  QArray<int> compositions;
  QArray<int> alphas;

  QString& compositionString(int);
  
public:
  LayerPanel(int n = 1);

  enum {Show = 0, Composition, Alpha, Tab};
  
  bool showing(int);
  int composition(int);
  int alpha(int);

  void setTopLeft(QPoint &);
  QRect rect(void);
  QSize size(void);
  int width(void) {return pix.width();}
  int height(void) {return pix.height();}
  int x(void) {return top_left_point.x();}
  int y(void) {return top_left_point.y();}
  QPixmap* pixmap(void);

  uint layerNum(int y);
  uint typeNum(int x);
  uint editing;

public slots:
  void setShowing(int, bool);
  void setShow(bool);
  void setComposition(int, int);
  void setAlpha(int, int);
  void setCurrent(int i) {current = i;}
  void setNumOfLayers(int n);

};

#endif 	// LAYERPANEL_H
