/**
 **	ptptformat.cpp
 **	PtptFormat
 **	by Saki Sakira <sakira@sun.dhis.portside.net>
 **	from 2003 August 5
 */

#include <stdio.h>
#include <qfile.h>
#include <qbuffer.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qpe/resource.h>

#include "ptptformat.h"

const QString PtptFormat::FileIconPath = "image_ptpt.png";
QImage PtptFormat::small_icon;
int PtptFormat::small_icon_width;
int PtptFormat::small_icon_height;
int PtptFormat::thumbnail_width;
int PtptFormat::thumbnail_height;
QString PtptFormat::dot_icon;

PtptFormat::PtptFormat(uint n) {
  init_small_icon();

  setNumOfLayers(n, true);
}

#if 0
PtptFormat::~PtptFormat(void) {
  for (uint k = 0; k < num_of_layers; k ++)
    if (layers[k])
      delete layers[k];
}
#endif

void PtptFormat::init_small_icon(void) {
  QImage img;

  if (img.load(Resource::findPixmap(FileIconPath))) {
    small_icon_width = img.width() * small_icon_height
      / img.height();
    small_icon.create(small_icon_width, small_icon_height, 32);

    int x, y, x2, y2, xx, yy;
    uint32 r, g, b;
    QRgb rgb = qRgb(0, 0, 0);
    uint32 ratio, ratio2;
    ratio = img.height() / small_icon_height;
    ratio2 = ratio * ratio;
    if (ratio2 == 0) ratio2 = 1;
    for (y = 0; y < small_icon_height; y ++) {
      y2 = y * img.height() / small_icon_height;
      for (x = 0; x < small_icon_width; x ++) {
        x2 = x * img.height() / small_icon_width;
        r = g = b = 0;
        for (yy = 0; yy < (int)ratio; yy ++)
          for (xx = 0; xx < (int)ratio; xx ++) {
            if (img.valid(x2 + xx, y2 + yy))
              rgb = img.pixel(x2 + xx, y2 + yy);
            r += qRed(rgb);
            g += qGreen(rgb);
            b += qBlue(rgb);
          }
        small_icon.setPixel(x, y,
                            qRgb(r / ratio2,
                                 g / ratio2,
                                 b / ratio2));
      }
    }
  } else
    qFatal("cannot load image %s", FileIconPath.latin1());
}

void PtptFormat::setNumOfLayers(uint n, bool reset) {
  num_of_layers = n;
  layer_infos.resize(n);
//  layers.resize(n);
  
  cur = 0;
  
  if (reset) {
    data.fill(0, 0x20 + 0x10 * n);
#if 0
    for (uint i = 0; i < n; i ++)
      layers[i] = 0;
#endif
  }
}

void PtptFormat::addLayer(QImage &img) {
  if (cur >= num_of_layers) return;
  
  QByteArray png;
  QBuffer buffer;
  QDataStream s;
  uint start_o, size;
  
  buffer.setBuffer(png);
  buffer.open(IO_WriteOnly);
  s.setDevice(&buffer);
  
  start_o = data.size();
  s << img;
  size = png.size();

  s.unsetDevice();
  buffer.close();

  data.resize(start_o + size);
  for (uint l = 0 ; l < size; l ++)
    data[start_o + l] = png[l];
  
  layer_infos[cur].offset = start_o;
  layer_infos[cur].length = size;

  cur ++;
}

void PtptFormat::setCompositionMethod(uint k, uint m) {
  if (k < num_of_layers)
    layer_infos[k].comp_method = m;
}

void PtptFormat::setAlpha(uint k, uint alpha) {
  if (k < num_of_layers)
    layer_infos[k].alpha = alpha;
}

void PtptFormat::setPaperColor(uint k, const QColor &col) {
  if (k < num_of_layers)
    layer_infos[k].paper_col = col;
}

void PtptFormat::setTNHeight(int height) {
  if (height == TNHeightSmall) {
    thumbnail_height = TNHeightSmall;
    small_icon_height = 15;
    dot_icon = QString(".icons");
  } else if (height == TNHeightLarge) {
    thumbnail_height = TNHeightLarge;
    small_icon_height = 30;
    dot_icon = QString(".icons144");
  } else {
    thumbnail_height = small_icon_height = 0;
    dot_icon = QString("");
  }
}

void PtptFormat::setThumbnail(const QImage &img) {
//  const uint ptpt_ptn[] = {
//    0xeeee, 0xa4a4, 0xe4e4, 0x8484};
  int height = thumbnail_height;
  int width = height * img.width() / img.height();

  if (width < small_icon_width) width = small_icon_width;
  thumbnail.create(width, height, 32);

  int x, y, x2, y2, xx, yy;
  uint32 r, g, b;
  QRgb rgb = qRgb(0, 0, 0);
  uint32 ratio, ratio2;
  ratio = img.height() / height;
  ratio2 = ratio * ratio;
  if (ratio2 == 0) ratio2 = 1;
  for (y = 0; y < height; y ++) {
    y2 = y * img.height() / height;
    for (x = 0; x < width; x ++) {
      x2 = x * img.width() / width;
      r = g = b = 0;
      for (yy = 0; yy < (int)ratio; yy ++)
        for (xx = 0; xx < (int)ratio; xx ++) {
          if (img.valid(x2 + xx, y2 + yy))
            rgb = img.pixel(x2 + xx, y2 + yy);
          r += qRed(rgb);
          g += qGreen(rgb);
          b += qBlue(rgb);
        }
      thumbnail.setPixel(x, y, 
                       qRgb(r / ratio2, g / ratio2, b /ratio2));
    }
  }

  int x_offset = width - small_icon_width;
  int y_offset = height - small_icon_height;
  for (y = 0; y < small_icon_height; y ++)
    for (x = 0; x < small_icon_width; x ++)
      thumbnail.setPixel(x + x_offset, y + y_offset,
                       small_icon.pixel(x, y));
  
#if 0
  for (y = 0; y < 4; y ++)
    for (x = 0; x < 16; x ++)
      thumbnail.setPixel(width - 16 + x, height - 4 + y,
                       (ptpt_ptn[y] & (0x8000 >> x)) ?
                       qRgb(0, 0, 0) : qRgb(255, 255,255));
#endif
}

bool PtptFormat::saveThumbnail(const QString &fp) {
  QFileInfo info(fp);
  QString dir, fn;

  dir = info.dirPath() + "/" + dot_icon + "/";
  fn = info.baseName() + "_" + info.extension() + ".jpg";

  QDir d(dir);
  if (!d.exists())
    d.mkdir(dir);

  return thumbnail.save(dir + fn, "JPEG");
}

bool PtptFormat::save(const QString &fn) {
  bool r;
  
  prepare_data();

  QFile f(fn);
  r = f.open(IO_WriteOnly);
  r &= (f.writeBlock(data) == (int)data.size());
  f.close();

  saveThumbnail(fn);

  return r;
}

void PtptFormat::prepare_data(void) {
  // file header
  
  data[0] = data[2] = 'P';
  data[1] = data[3] = 't';
  data[4] = 0;
  data[5] = '1'; data[6] = '.'; data[7] = '0';

  // infomation header

  data[0x10] = num_of_layers;

  for (uint k = 0; k < num_of_layers; k ++) {
    for (uint j = 0; j < 4; j ++) {
      data[0x20 + k * 0x10 + j]
        = (layer_infos[k].offset >> (24 - 8*j)) & 0xff;
      data[0x24 + k * 0x10 + j]
        = (layer_infos[k].length >> (24 - 8*j)) & 0xff;
    }

    data[0x28 + k * 0x10] = layer_infos[k].comp_method;
    data[0x29 + k * 0x10] = layer_infos[k].alpha;
    data[0x2a + k * 0x10] = layer_infos[k].paper_col.red();
    data[0x2b + k * 0x10] = layer_infos[k].paper_col.green();
    data[0x2c + k * 0x10] = layer_infos[k].paper_col.blue();
  }
}

bool PtptFormat::isPtpt(const QString &fn) {
  if (!QFile::exists(fn))
    return false;
  
  char buf[9];
  QFile f(fn);
  buf[8] = 0;

  f.open(IO_ReadOnly);
  f.readBlock(buf, 8);
  f.close();

  if (QString(buf) == "PtPt" &&
      buf[5] >= '0' && buf[5] <= '9' &&
      buf[6] == '.' &&
      buf[7] >= '0' && buf[7] <= '9')
    return true;
  else
    return false;
}

bool PtptFormat::isPtpt(void) {
  if (QString(data.data()) == "PtPt" &&
      data[5] >= '0' && data[5] <= '9' &&
      data[6] == '.' &&
      data[7] >= '0' && data[7] <= '9')
    return true;
  else
    return false;
}

void PtptFormat::parse_data(void) {
  if (!isPtpt()) return;

  // infomation header

  uchar r, g, b;
  
  setNumOfLayers(data[0x10]);

  uint32 offset, length;
  for (uint k = 0; k < num_of_layers; k ++) {
    offset = length = 0;
    for (uint j = 0; j < 4; j ++) {
      offset = (offset << 8) +
        (uchar)data[0x20 + k * 0x10 + j];
      length = (length << 8) +
        (uchar)data[0x24 + k * 0x10 + j];
    }
    layer_infos[k].offset = offset;
    layer_infos[k].length = length;
//    printf("%d %d %d\n", k, offset, length);

    layer_infos[k].comp_method = data[0x28 + k * 0x10];
    layer_infos[k].alpha = data[0x29 + k * 0x10];

    r = (uchar)data[0x2a + k * 0x10];
    g = (uchar)data[0x2b + k * 0x10];
    b = (uchar)data[0x2c + k * 0x10];
    layer_infos[k].paper_col.setRgb(r, g, b);
  }
}

uint PtptFormat::numOfLayers(void) {
  return num_of_layers;
}

QImage PtptFormat::layer(uint k) {
  QImage img;

  img.loadFromData(
    ((uchar*)data.data()) + layer_infos[k].offset,
    layer_infos[k].length);

  return img;
}

uint PtptFormat::compositionMethod(uint k) {
  return layer_infos[k].comp_method;
}

uint PtptFormat::alpha(uint k) {
  return layer_infos[k].alpha;
}

QColor PtptFormat::paperColor(uint k) {
  return layer_infos[k].paper_col;
}

bool PtptFormat::load(const QString &fn) {
  uint len;
  char * buf;
  QFile f(fn);

  len = QFileInfo(f).size();
  buf = new char[len];

  f.open(IO_ReadOnly);
  f.readBlock(buf, len);
  f.close();

  data.resize(len);
  data.duplicate(buf, len);
  delete buf;

  parse_data();

  return true;
}
