/*
 **  simplepaint.cpp 
 **	by Saki Sakira <sakira@sun.dhis.portside.net>
 **	from 2003 June 7
 */

//#include <qcolordialog.h>
#include <stdio.h>
#include <qvbox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qdirectpainter_qws.h>
#include <qarray.h>
#include <qimage.h>
#include <qmenubar.h>
#include <qstring.h>
#include <qpopupmenu.h>
//#include <qfiledialog.h>
//#include <slfileselector.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qdir.h>

#include "filedialog.h"

#include "constants.h"
#include "canvas.h"
#include "toolpanel.h"
#include "ptptformat.h"
#include "layeredpainter.h"
#include "tspressure.h"
#include "colorpalette.h"
#include "spainter.h"
//#include "toolbar.h"

////////// SPainter

#include "version.h"
const QString SPainter::VERSION(PTPT_VERSION);

//SPainter::SPainter(QWidget *parent)
//     : QMainWindow(parent, 0, WStyle_Customize | WStyle_NoBorder)
SPainter::SPainter(QWidget *parent)
     : QMainWindow(parent, 0,WStyle_Customize | WStyle_NoBorder)
{
  // add by anoda 2003.10.27
  flgx = 0;

  setCaption(QString::null);

//  editing_fn = QDir::homeDirPath() + "/.";
  prepareFilename();

  tools = 0;
  colorpanel = 0;

  set_env_depend();

  canvas = new Canvas(this);
  load_settings();

  setup_menu();
  //  setup_toolbar();

  ColorPalette::setSettings(&settings);

  if (tspressure.pressure() < 0) {
    canvas->applyPressureToWidth(false);
    canvas->applyPressureToDensity(false);
  }

  setCentralWidget(canvas);

  set_user_palette();
}

SPainter::~SPainter() {
  delete canvas;
  delete file_menu;
  delete layer_menu;
  delete other_menu;
  delete menubar;
}

// add by anoda 2003.10.27
void SPainter::ldx()
{
  static int flg = 0;
  if ((flg == 0) && (flgx)){

    canvas->load(fnx);

    QFileInfo info(fnx);
    editing_fn = info.dirPath() + "/" + info.baseName();
    settings.setLastFileName(editing_fn);
  }
  flg = 1;
}

void SPainter::setFNX(const char * buf)
{
  fnx = buf;
  flgx = 1;
}
 
void SPainter::setDocument(const QString& buf)
{
  fnx = buf;
  flgx = 1;
}

void SPainter::set_env_depend(void) {
  QDirectPainter dp(this);
  int l = dp.lineStep();

  if (l == 480 * 2) {
    // SL-C7x0 ?
    use_sl = true;
    PtptFormat::setTNHeight(PtptFormat::TNHeightLarge);
  } else if (l == 320 * 2) {
    // SL-B500 ?
    use_sl = true;
    PtptFormat::setTNHeight(PtptFormat::TNHeightSmall);
  } else {
    // SL-A300 or iPaq?
    use_sl = false;
    PtptFormat::setTNHeight(PtptFormat::TNHeightSmall);
  }
}

void SPainter::keyPressEvent(QKeyEvent *e) {
  canvas->keyPressEvent(e);
}

void SPainter::setup_menu(void) {
  menubar = menuBar();

  QPopupMenu *ptpt_menu = new QPopupMenu(this, "PtPt");
  ptpt_menu->insertItem(QString("Hide Menubar"), canvas,
                        SLOT(show_hide_menu()));
  ptpt_menu->insertSeparator();
  ptpt_menu->insertItem(QString("About..."), this,
                         SLOT(show_about()));
  ptpt_menu->insertSeparator();
  ptpt_menu->insertItem(QString("New Image"), this,
                        SLOT(clearImage()));
  ptpt_menu->insertSeparator();
  ptpt_menu->insertItem(QString("Quit..."), canvas,
                        SLOT(quit_application()));
  menubar->insertItem(QString("PtPt"), ptpt_menu);

  file_menu = new QPopupMenu(this, "File");
  file_menu->insertSeparator();
  file_menu->insertItem(QString("Load Image..."), this,
                        SLOT(loadImage()));
  file_menu->insertItem(QString("Save Image"), this,
                        SLOT(save()));
  file_menu->insertItem(QString("Save Image As..."), this,
                        SLOT(saveImage()));
  file_menu->insertSeparator();
  file_menu->insertItem(QString("Save PNG..."), this,
                        SLOT(savePNG()));
  file_menu->insertItem(QString("Save JPEG..."), this,
                        SLOT(saveJPG()));
  file_menu->insertSeparator();
  file_menu->insertItem(QString("Load to Current Layer..."),
                         this, SLOT(loadLayer()));
  file_menu->insertItem(QString("Save Current Layer..."),
                         this, SLOT(saveLayer()));
  file_menu->insertSeparator();
  file_menu->insertItem("Refresh Icons...",
                        this, SLOT(refreshIcons(void)));
  menubar->insertItem(QString("File"), file_menu);

  edit_menu = new QPopupMenu(this, "Edit");
  edit_menu->insertItem("Undo", canvas,
			SLOT(undo()));  
  edit_menu->insertSeparator();
  edit_menu->insertItem("Copy Rectangle", canvas,
			SLOT(set_select_mode_copy()));
  edit_menu->insertItem("Paste Mode", canvas,
			SLOT(set_select_mode_paste()));
  edit_menu->insertItem("Paste Freeze", canvas,
			SLOT(set_select_mode_paste_freeze()));
  edit_menu->insertItem("Paste Cancel", canvas,
			SLOT(set_select_mode_paste_cancel()));
  edit_menu->insertSeparator();
  edit_menu->insertItem("Clear Clipboard", canvas,
			SLOT(clearClipboard()));
  menubar->insertItem("Edit", edit_menu);

  QPopupMenu *comp_menu = new QPopupMenu(this, "Composition");
  comp_menu->insertItem(QString("Min. Composition"), canvas,
                        SLOT(setMinComposition()));
  comp_menu->insertItem(QString("Max. Composition"), canvas,
                        SLOT(setMaxComposition()));
  comp_menu->insertItem(QString("Mul. Composition"), canvas,
                        SLOT(setMulComposition()));
  comp_menu->insertItem(QString("Screen Composition"), canvas,
                        SLOT(setScreenComposition()));
  comp_menu->insertItem(QString("Sat. Composition"), canvas,
                         SLOT(setSatComposition()));
  comp_menu->insertItem(QString("Col. Composition"), canvas,
                         SLOT(setColComposition()));
  comp_menu->insertItem(QString("Normal Composition"), canvas,
                         SLOT(setNormalComposition()));
  comp_menu->insertItem("Mask Composition", canvas,
			 SLOT(setMaskComposition()));
  comp_menu->insertItem("Alpha Ch. Composition", canvas,
			 SLOT(setAlphaChannelComposition()));
  menubar->insertItem(QString("Composition"), comp_menu);

  layer_menu = new QPopupMenu(this, "Layer");
  layer_menu->insertItem("Move Up Current Layer", canvas,
                         SLOT(moveUpCurrentLayer()));
  layer_menu->insertItem("Move Down Current Layer", canvas,
                         SLOT(moveDownCurrentLayer()));
  layer_menu->insertSeparator();
  layer_menu->insertItem("Scale Current Layer", this,
                         SLOT(scaleLayer()));
  layer_menu->insertItem(QString("Clear Current Layer"), canvas,
                         SLOT(clearCurrentLayer()));
  layer_menu->insertItem(QString("Set Paper Color..."), canvas,
                         SLOT(setPaperColor()));
  layer_menu->insertSeparator();
  layer_menu->insertItem("Add New Layer", canvas,
                         SLOT(addNewLayer()));
  layer_menu->insertItem("Dup. Current Layer", canvas,
			 SLOT(duplicateCurrentLayer()));
  layer_menu->insertItem("Delete Current Layer", canvas,
                         SLOT(deleteCurrentLayer()));
  layer_menu->insertItem(QString("Join Showing Layers"), canvas,
                        SLOT(joinShowingLayers()));
  menubar->insertItem(QString("Layer"), layer_menu);

  tool_menu = new QPopupMenu(this, "Tool");
  tool_menu->insertItem("Mirror Horizontal", canvas,
                        SLOT(mirrorHorizontal()));
  tool_menu->insertItem("Mirror Vertical", canvas,
                        SLOT(mirrorVertical()));
  tool_menu->insertSeparator();
  tool_menu->insertItem("Resize Canvas", this,
                        SLOT(setCanvasSize()));
  tool_menu->insertSeparator();
  tool_menu->insertItem("Rotate CW", canvas,
                        SLOT(rotateCW()));
  tool_menu->insertItem("Rotate CCW", canvas,
                        SLOT(rotateCCW()));
  menubar->insertItem(QString("Tool"), tool_menu);
  
  other_menu = new QPopupMenu(this, "Other");
  other_menu->insertItem("Set Palette...", this,
                         SLOT(setPalette()));
  other_menu->insertItem("Init Palette", this,
                         SLOT(initPalette()));
  other_menu->insertSeparator();
  other_menu->insertItem("Pen Pressure...", this,
                         SLOT(penPressure()));
  other_menu->insertItem("Settings...", this,
                         SLOT(showSettings()));
  other_menu->insertSeparator();
  other_menu->insertItem(QString("Hide Menubar"), canvas,
                        SLOT(show_hide_menu()));
  menubar->insertItem(QString("Other"), other_menu);

  menubar->hide();
}

#if 0
void SPainter::setup_toolbar(void) {
  toolbar = new PPToolBar(this);
  setToolBarsMovable(false);
  //  addToolBar(toolbar, Top);
  toolbar->hide();
  toolbar->setCanvas(canvas);
}
#endif

bool SPainter::noviceMode(void) {
  return settings.getNoviceMode();
}

void SPainter::load_settings(void) {
  TSPressure::press_dist news_min, news_max;

  TSPressure::setPressureMin(settings.getPressure(
    Settings::i_pressure_min));
  TSPressure::setPressureMid(settings.getPressure(
    Settings::i_pressure_mid));
  TSPressure::setPressureMax(settings.getPressure(
    Settings::i_pressure_max));
  TSPressure::setSmooth(settings.getPressureSmooth());

  news_min.oo = settings.getPressure(Settings::i_pressure_min);
  news_min.po = settings.getPressure(Settings::i_pressure_east_min);
  news_min.mo = settings.getPressure(Settings::i_pressure_west_min);
  news_min.op = settings.getPressure(Settings::i_pressure_south_min);
  news_min.om = settings.getPressure(Settings::i_pressure_north_min);
  news_max.oo = settings.getPressure(Settings::i_pressure_max);
  news_max.po = settings.getPressure(Settings::i_pressure_east_max);
  news_max.mo = settings.getPressure(Settings::i_pressure_west_max);
  news_max.op = settings.getPressure(Settings::i_pressure_south_max);
  news_max.om = settings.getPressure(Settings::i_pressure_north_max);
  TSPressure::setPressureMin(news_min);
  TSPressure::setPressureMax(news_max);

  canvas->setSyncWidth(settings.getSyncWidth());
  if (settings.getLeftRight())
    canvas->setLeftRight(Canvas::LeftRightDensity);
  else
    canvas->setLeftRight(Canvas::LeftRightSwitchMethod);
  canvas->setSeparatePen(settings.getSeparatePen());

  canvas->adjustColorTable(settings.getZColorAdjust());
}

void SPainter::setShowMenu(bool s) {
  if (s) {
    menubar->show();
    //    toolbar->show();
  } else {
    menubar->hide();
    //    toolbar->hide();
    enableFullscreen();
  }
}

void SPainter::resizeEvent(QResizeEvent *) {
  if (size() != qApp->desktop()->size())
    enableFullscreen();
}

void SPainter::focusInEvent(QFocusEvent *) {
  enableFullscreen();
}

void SPainter::enableFullscreen(void) {
  setFixedSize(qApp->desktop()->size());
  showNormal();
  reparent(0, WType_TopLevel | WStyle_Customize |
           WStyle_NoBorderEx, QPoint(0, 0));
//  reparent(0, WStyle_Customize | WStyle_NoBorder,
//           QPoint(0, 0));
  showFullScreen();
}

void SPainter::setFullScreen(void) {
  if (size() != qApp->desktop()->size())
    enableFullscreen();
}

#if 0
void SPainter::setFullScreen(void) {
  int h = QApplication::desktop()->height();
  int w = QApplication::desktop()->width();

  if (height() == h)
    return;

  showNormal();
  setFixedHeight(h);
  setFixedWidth(w);
  reparent(0, WType_TopLevel | WStyle_Customize |
           WStyle_NoBorderEx, QPoint(0, 0));
  showFullScreen();
}
#endif

void SPainter::setPalette(void) {
  QString fn = settings.getPaletteFileName();
  FileDialog dlg("load palette", true, fn);

  if (dlg.exec()) {
    fn = dlg.filePath();
    settings.setPaletteFileName(fn);
    set_user_palette();
  }
}

void SPainter::initPalette(void) {
  QFileInfo info(settings.getPaletteFileName());
  QString fn = info.dirPath();
  settings.setPaletteFileName(Resource::findPixmap(ColorPalette::Filename));
  set_user_palette();
  
//  settings.setPalleteFileName(fn + "/.");
//  tools->initPallete();
}

void SPainter::penPressure(void) {
  TSDialog d;

  d.setMinPressure(
    settings.getPressure(Settings::i_pressure_min));
  d.setMidPressure(
    settings.getPressure(Settings::i_pressure_mid));
  d.setMaxPressure(
    settings.getPressure(Settings::i_pressure_max));

  d.setMinPressure(TSDialog::NEWSEast,
		   settings.getPressure(Settings::i_pressure_east_min));
  d.setMinPressure(TSDialog::NEWSWest,
		   settings.getPressure(Settings::i_pressure_west_min));
  d.setMinPressure(TSDialog::NEWSSouth,
		   settings.getPressure(Settings::i_pressure_south_min));
  d.setMinPressure(TSDialog::NEWSNorth,
		   settings.getPressure(Settings::i_pressure_north_min));
  d.setMaxPressure(TSDialog::NEWSEast,
		   settings.getPressure(Settings::i_pressure_east_max));
  d.setMaxPressure(TSDialog::NEWSWest,
		   settings.getPressure(Settings::i_pressure_west_max));
  d.setMaxPressure(TSDialog::NEWSSouth,
		   settings.getPressure(Settings::i_pressure_south_max));
  d.setMaxPressure(TSDialog::NEWSNorth,
		   settings.getPressure(Settings::i_pressure_north_max));

  d.setSmooth(settings.getPressureSmooth());
  d.showValues();

  if (d.exec()) {
    unsigned long int minp, midp, maxp, smooth;
    unsigned long int east_minp, east_maxp, west_minp, west_maxp,
      south_minp, south_maxp, north_minp, north_maxp;
    TSPressure::press_dist news_min, news_max;

    minp = d.minPressure();
    midp = d.midPressure();
    maxp = d.maxPressure();
    smooth = d.smooth();
    east_minp = d.minPressure(TSDialog::NEWSEast);
    west_minp = d.minPressure(TSDialog::NEWSWest);
    south_minp = d.minPressure(TSDialog::NEWSSouth);
    north_minp = d.minPressure(TSDialog::NEWSNorth);
    east_maxp = d.maxPressure(TSDialog::NEWSEast);
    west_maxp = d.maxPressure(TSDialog::NEWSWest);
    south_maxp = d.maxPressure(TSDialog::NEWSSouth);
    north_maxp = d.maxPressure(TSDialog::NEWSNorth);

    news_min.oo = minp;
    news_min.po = east_minp;
    news_min.mo = west_minp;
    news_min.op = south_minp;
    news_min.om = north_minp;
    news_max.oo = maxp;
    news_max.po = east_maxp;
    news_max.mo = west_maxp;
    news_max.op = south_maxp;
    news_max.om = north_maxp;
    
    TSPressure::setPressureMin(minp);
    TSPressure::setPressureMid(midp);
    TSPressure::setPressureMax(maxp);
    TSPressure::setSmooth(smooth);
    TSPressure::setPressureMin(news_min);
    TSPressure::setPressureMax(news_max);

    settings.setPressure(Settings::i_pressure_min, minp);
    settings.setPressure(Settings::i_pressure_mid, midp);
    settings.setPressure(Settings::i_pressure_max, maxp);

    settings.setPressure(Settings::i_pressure_east_min, east_minp);
    settings.setPressure(Settings::i_pressure_west_min, west_minp);
    settings.setPressure(Settings::i_pressure_south_min, south_minp);
    settings.setPressure(Settings::i_pressure_north_min, north_minp);
    settings.setPressure(Settings::i_pressure_east_max, east_maxp);
    settings.setPressure(Settings::i_pressure_west_max, west_maxp);
    settings.setPressure(Settings::i_pressure_south_max, south_maxp);
    settings.setPressure(Settings::i_pressure_north_max, north_maxp);

    settings.setPressureSmooth(smooth);

    settings.save();
  }
}

void SPainter::showSettings(void) {
  SettingsDialog d;

  d.setSyncWidth(settings.getSyncWidth());
  d.setLeftRight(settings.getLeftRight());
  d.setSeparatePen(settings.getSeparatePen());
  d.setNoviceMode(settings.getNoviceMode());
  d.setZColorAdjust(settings.getZColorAdjust());
  
  if (d.exec()) {
    canvas->setSyncWidth(d.syncWidth());
    if (d.leftRight())
      canvas->setLeftRight(Canvas::LeftRightDensity);
    else
      canvas->setLeftRight(Canvas::LeftRightSwitchMethod);
    canvas->setSeparatePen(d.separatePen());
    canvas->adjustColorTable(d.ZColorAdjust());
    
    settings.setSyncWidth(d.syncWidth());
    settings.setLeftRight(d.leftRight());
    settings.setSeparatePen(d.separatePen());
    settings.setNoviceMode(d.noviceMode());
    settings.setZColorAdjust(d.ZColorAdjust());
    settings.save();
  }

}

void SPainter::clearImage(void) {
  if (QMessageBox::warning(this, "PetitePeinture",
                           "Discard current image?\n\n",
                           "New Image", "Cancel"))
    return;

  canvas->clearImage();
  menubar->hide(); menubar->show();
  prepareFilename();
  image_saved = false;
}

void SPainter::prepareFilename(bool saved) {
  QFileInfo info(settings.getLastFileName());
  QString fn = info.dirPath() + "/" + info.baseName();
  editing_fn = fn;

  QString tail, num;
  int tail_num;
  while (LayeredPainter::fileExists(fn) ||
         QFile::exists(fn + ".ptpt") ||
         QFile::exists(fn + ".png") ||
         QFile::exists(fn + ".jpg") ||
         QFile::exists(fn)) {
    tail = fn.right(4);
    tail_num = tail.right(3).toInt();
    if ((tail[0] != QChar('_')) ||
        ("_" + num.sprintf("%03d", tail_num) != tail)) {
      fn += "_000";
    } else {
      if (tail_num < 999)
        fn = fn.left(fn.length() - 3) +
          num.sprintf("%03d", tail_num + 1);
      else
        fn += "_000";
    }
  }

  if (fn != editing_fn) {
    editing_fn = fn;
    settings.setLastFileName(editing_fn);
  }

  image_saved = saved;
}

void SPainter::loadImage(void) {
  editing_fn = settings.getLastFileName();
  FileDialog dlg("load image", TRUE, editing_fn);

  if (dlg.exec()) {
    QString fn = dlg.filePath();
    canvas->load(fn);

    QFileInfo info(fn);
    editing_fn = info.dirPath() + "/" + info.baseName();
    settings.setLastFileName(editing_fn);
  }
}

void SPainter::loadLayer(void) {
  editing_fn = settings.getLastFileName();
  FileDialog dlg("load to current layer",
                 TRUE, editing_fn);

  if (dlg.exec()) {
    QString fn = dlg.filePath();
    canvas->loadLayer(fn);

#if 0
    QFileInfo info(fn);
    editing_fn = info.dirPath() + "/" + info.baseName();
    settings.setLastFileName(editing_fn);
#endif
  }
}

void SPainter::save(void) {
  if (image_saved) {
    QString fn = editing_fn;

    if (fn.right(5) != ".ptpt")
      fn += ".ptpt";

    canvas->save(fn, "PTPT", false);
  } else
    saveImage();
}

void SPainter::saveImage(void) {
  editing_fn = settings.getLastFileName();
  FileDialog dlg("save image", FALSE, editing_fn);

  if (dlg.exec()) {
    QString fn = dlg.filePath();
    settings.setLastFileName(fn);

    editing_fn = fn;

    if (fn.right(5) != ".ptpt")
      fn += ".ptpt";

    canvas->save(fn, "PTPT");
  }

  image_saved = true;
}

void SPainter::savePNG(void) {
  editing_fn = settings.getLastFileName();
  FileDialog dlg("save PNG image", FALSE, editing_fn);

  if (dlg.exec()) {
    QString fn = dlg.filePath();
    editing_fn = fn;
    settings.setLastFileName(fn);

    if (fn.right(4) != ".png")
      fn += ".png";

    canvas->save(fn, "PNG");
  }
}

void SPainter::saveJPG(void) {
  editing_fn = settings.getLastFileName();
  FileDialog dlg("save JPEG image", FALSE, editing_fn);

  if (dlg.exec()) {
    QString fn = dlg.filePath();
    editing_fn = fn;
    settings.setLastFileName(fn);

    if (fn.right(4) != ".jpg")
      fn += ".jpg";

    canvas->save(fn, "JPEG");
  }
}

void SPainter::saveLayer(void) {
  editing_fn = settings.getLastFileName();
  FileDialog dlg("save current layer", FALSE, editing_fn);

  if (dlg.exec()) {
    QString fn = dlg.filePath();
#if 0
    editing_fn = fn;
    settings.setLastFileName(fn);
#endif

    if (fn.right(4) != ".png")
      fn += ".png";

    canvas->saveLayer(fn);
  }
}

void SPainter::refreshIcons(void) {
  if (QMessageBox::warning(this, "PetitePeinture",
                           "Discard current image?\n\n",
                           "Refresh Icons", "Cancel"))
    return;

  editing_fn = settings.getLastFileName();
  FileDialog dlg("refresh icons", true, editing_fn);

  if (dlg.exec()) {
    QFileInfo info(dlg.filePath());
    QDir dir = info.dir();
    dir.setNameFilter("*.ptpt");
    QStringList fns = dir.entryList();

    QString fn;
    for (uint i = 0; i < fns.count(); i ++) {
      fn = dir.path() + "/" + fns[i];
      if (PtptFormat::isPtpt(fn)) {
        canvas->load(fn);
        canvas->saveIcon(fn);
      }
    }
  
    canvas->clearImage();
    prepareFilename();
  }
}

void SPainter::setTools(ToolPanel *t) {
  tools = t;

  set_user_palette();
}

void SPainter::setColorPanel(ColorPanel *c) {
  colorpanel = c;

  set_user_palette();
}

void SPainter::set_user_palette(void) {
  if (tools)
    tools->setColorPalette(settings.getPaletteFileName());
  if (colorpanel)
    colorpanel->setUserPalette(settings.getPaletteFileName());
}

void SPainter::show_about(void) {
  QMessageBox::information(this, QString("PetitePeinture "),
                           "PetitePeinture \n Version "
                           + VERSION + "\n"
                           "maintained by Saki Sakira\n"
                           " <sakira@sun.dhis.portside.net>");
}

void SPainter::scaleLayer(void) {
  uint w, h;

  w = canvas->canvasWidth();
  h = canvas->canvasHeight();
  ScaleDialog d(w, h);

  if (d.exec()) {
    canvas->scaleLayer(d.x(), d.y());
  }
}

void SPainter::setCanvasSize(void) {
  uint w, h;

  w = canvas->canvasWidth();
  h = canvas->canvasHeight();
  SetCanvasSizeDialog d(w, h);

  if (d.exec())
    canvas->setSize(d.w(), d.h());
}

void SPainter::quit_application(void) {
  //  qApp->quit();
  close();
}

////////////////////////////////////////////////////////////
////	ScaleDialog

#include <qdialog.h>
#include <qgrid.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qradiobutton.h>

ScaleDialog::ScaleDialog(uint sx, uint sy) :
QDialog(0, "ScaleDialog", true) {
  QString sxs, sys;

  setCaption("Scaling");
  sxs.setNum(sx);
  sys.setNum(sy);
  width_ = sx;
  height_ = sy;
  btns.setRadioButtonExclusive(true);

  main_box = new QGrid(3, this);
  main_box->setMinimumSize(240, 200);
  new QLabel("", main_box);
  pixel_cb = new QRadioButton("Pixel", main_box);
  percent_cb = new QRadioButton("Ratio(%)", main_box);
  btns.insert(pixel_cb);
  btns.insert(percent_cb);
  x_label = new QLabel("X: ", main_box);
  x_edit = new QLineEdit(sxs, main_box);
  xp_edit = new QLineEdit("100", main_box);
  y_label = new QLabel("Y: ", main_box);
  y_edit = new QLineEdit(sys, main_box);
  yp_edit = new QLineEdit("100", main_box);

  pixel_cb->setChecked(true);
}

ScaleDialog::~ScaleDialog(void) {
  delete y_edit;
  delete y_label;
  delete yp_edit;
  delete x_edit;
  delete x_label;
  delete xp_edit;
  delete main_box;
}

uint ScaleDialog::x(void) {
  if (pixel_cb->isChecked())
    return x_edit->text().toUInt();
  else
    return xp_edit->text().toUInt() * width_ / 100;
}

uint ScaleDialog::y(void) {
  if (pixel_cb->isChecked())
    return y_edit->text().toUInt();
  else
    return yp_edit->text().toUInt() * height_ / 100;
}

////////////////////////////////////////////////////////////
//	SetCanvasSizeDialog

SetCanvasSizeDialog::SetCanvasSizeDialog(uint w, uint h) :
  QDialog(0, "SetCanvasSizeDialog", true) {
  
  setCaption("Resize Canvas");
  width_ = w;
  height_ = h;

  main_box = new QGrid(2, this);
  main_box->setMinimumSize(200, 200);
  w_label = new QLabel("width: ", main_box);
  w_edit = new QLineEdit(QString::number(w), main_box);
  h_label = new QLabel("height: ", main_box);
  h_edit = new QLineEdit(QString::number(h), main_box);
}

SetCanvasSizeDialog::~SetCanvasSizeDialog(void) {
  delete w_label;
  delete h_label;
  delete w_edit;
  delete h_edit;
  delete main_box;
}

uint SetCanvasSizeDialog::w(void) {
  return w_edit->text().toUInt();
}

uint SetCanvasSizeDialog::h(void) {
  return h_edit->text().toUInt();
}
