#ifndef SPAINTER_H
#define SPAINTER_H

#include <qmainwindow.h>
#include <qwidget.h>
#include <qvbox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpointarray.h>
#include <qlist.h>
#include <qcolordialog.h>
#include <qrect.h>
#include <qimage.h>
#include <qmenubar.h>

#include "settings.h"
#include "tspressure.h"

class Canvas;
class ToolPanel;
class ColorPanel;
class PPToolBar;

class SPainter : public QMainWindow {
  Q_OBJECT
    
public:
  static const QString VERSION;

  SPainter(QWidget *parent = 0);
  ~SPainter();

  void keyPressEvent(QKeyEvent*);
  void focusInEvent(QFocusEvent *);
  void resizeEvent(QResizeEvent *);
  void enableFullscreen(void);
  void setShowMenu(bool);
  void setTools(ToolPanel *);
  void setColorPanel(ColorPanel *);
  void prepareFilename(bool = false);
  int menuHeight(void) {return menubar->height();}
  bool noviceMode(void);
  
  // add by anoda 2003.10.27
  void ldx();
  void setFNX(const char *);
  
private:
  Canvas *canvas;
  QMenuBar *menubar;
  //  PPToolBar *toolbar;
  QPopupMenu *file_menu, *edit_menu, *layer_menu, *tool_menu, *other_menu;
  QString editing_fn;
  Settings settings;
  ToolPanel *tools;
  ColorPanel *colorpanel;
  TSPressure tspressure;
  bool use_sl;
  bool image_saved;

  void set_env_depend(void);
  void setup_menu(void);
  //  void setup_toolbar(void);
  void load_settings(void);
  void set_user_palette(void);

  // add by anoda 2003.10.27
  QString fnx;
  int flgx;

public slots:
  void clearImage(void);
  void loadImage(void);
  void save(void);
  void saveImage(void);
  void refreshIcons(void);
  void savePNG(void);
  void saveJPG(void);
  void loadLayer(void);
  void saveLayer(void);
  void quit_application(void);
  void show_about(void);
  void setFullScreen(void);
  void setPalette(void);
  void initPalette(void);
  void penPressure(void);
  void showSettings(void);
  void scaleLayer(void);
  void setCanvasSize(void);
  
  // add by anoda 2003.10.27
  void setDocument(const QString&);
};

////////////////////////////////////////////////////////////
//	ScaleDialog

#include <qdialog.h>
#include <qbuttongroup.h>
class QRadioButton;
class QGrid;
class QLineEdit;
class QLabel;

class ScaleDialog : public QDialog {
  QGrid *main_box;
  QLabel *x_label, *y_label, *xp_label, *yp_label;
  QLineEdit *x_edit, *y_edit, *xp_edit, *yp_edit;
  QButtonGroup btns;
  QRadioButton *pixel_cb, *percent_cb;
  uint width_, height_;
  
public:
  ScaleDialog(uint, uint);
  ~ScaleDialog(void);

  uint x(void);
  uint y(void);
};

////////////////////////////////////////////////////////////
//	SetCanvasSizeDialog

class SetCanvasSizeDialog : public QDialog {
  QGrid *main_box;
  QLabel *w_label, *h_label;
  QLineEdit *w_edit, *h_edit;

  uint width_, height_;

 public:
  SetCanvasSizeDialog(uint, uint);
  ~SetCanvasSizeDialog(void);

  uint w(void);
  uint h(void);
};

#endif // SPAINTER_H
