/*
**	PPToolBar
**	by Saki Sakira <sakira@sun.dhis.portside.net>
**	from February 7, 2004
*/

#include <qcombobox.h>

#include "canvas.h"
#include "toolbar.h"

PPToolBar::PPToolBar(QMainWindow *parent) : QToolBar(parent) {
  this->parent = parent;

  QComboBox** cboxes[4] = {&cloud_btn, &eraser_btn, &water_btn, &pencil_btn};
  char titles[] = "CEWP";
  QString title;

  for (int b = 0; b < 4; b ++) {
    *cboxes[b] = new QComboBox(this);
    title = QChar(titles[b]);
    for (int w = 0; w < PenWidthMax; w ++)
      (*cboxes[b])->insertItem(title + QString::number(w));
  }
}

PPToolBar::~PPToolBar(void) {
  delete pencil_btn;
  delete water_btn;
  delete eraser_btn;
  delete cloud_btn;
}

void PPToolBar::setCanvas(Canvas *c) {
  canvas = c;

  connect(pencil_btn, SIGNAL(activated(int)),
	  c, SLOT(setPencil(int)));
  connect(water_btn, SIGNAL(activated(int)),
	  c, SLOT(setBrush(int)));
  connect(eraser_btn, SIGNAL(activated(int)),
	  c, SLOT(setEraser(int)));
  connect(cloud_btn, SIGNAL(activated(int)),
	  c, SLOT(setCloud(int)));

  connect(canvas, SIGNAL(pencilWidthChanged(int)),
	  this, SLOT(setPencilWidth(int)));
  connect(canvas, SIGNAL(brushWidthChanged(int)),
	  this, SLOT(setWaterWidth(int)));
  connect(canvas, SIGNAL(eraserWidthChanged(int)),
	  this, SLOT(setEraserWidth(int)));
  connect(canvas, SIGNAL(cloudWidthChanged(int)),
	  this, SLOT(setCloudWidth(int)));
}

void PPToolBar::setPencilWidth(int w) {
  pencil_btn->setCurrentItem(w);
}

void PPToolBar::setWaterWidth(int w) {
  water_btn->setCurrentItem(w);
}

void PPToolBar::setEraserWidth(int w) {
  eraser_btn->setCurrentItem(w);
}

void PPToolBar::setCloudWidth(int w) {
  cloud_btn->setCurrentItem(w);
}
