#ifndef TOOLPANEL_H
#define TOOLPANEL_H

#include <qstring.h>
#include <qarray.h>
#include <qrect.h>
#include "ptpen.h"

class QSize;
class SketchPainter;
class ColorPalette;

class ToolPanel {
  unsigned short int *buf;
  SketchPainter *sp;
  int width, height;
  double ratio;
  PtPen pen;
  int density;
  ColorPalette *color_palette;

  QArray<QRect> tools;
  QArray<bool> active;
  QArray<bool> layers;

public:

  static const uint num_of_layers = 6;
  
  enum {Palette = 0,
    Undo, Menu, Select, Move, Zoom,
    Cloud, Eraser, Brush, Pencil,
    PenInfo, Layer,
	SelectL, SelectR, MoveL, MoveR,
    ZoomL, ZoomR, CloudL, CloudR,
    EraserL, EraserR, BrushL, BrushR,
	PencilL, PencilR, 
    Layers};
  static const uint NumOfTools = Layers + num_of_layers;

  ToolPanel(QString &, int = 480);
  ToolPanel(int = 480);
  ~ToolPanel();

  static const QString ToolPanelFileName;

  QSize size(void);
  int getIndex(int x, int y);
  bool isLeftside(int, int);
  QColor getColor(int, int);
  void setActive(int, bool);
  void clearSelection(void);
  void clearSelectionHalf(void);
  void setPen(const PtPen &);
  void setPenDensity(int);
  void setPenMethod(int);
  void setShowLayer(int, bool);
  void setColorPalette(const QString & = "");
  void initPalette(void);
  QRect rect(int i) {return tools[i];};

  unsigned short int* infoImage(void);

private:
  void initialize(QString &, int);
  void setup_tools(void);

  void copy_buf(void);
  
};

#endif		//  TOOLPANEL_H
