#ifndef TSPRESSURE_H
#define TSPRESSURE_H

#include <qdialog.h>
#include <qwidget.h>

#include "constants.h"

class TSPressure {
  static const char* DevFN;
  static const unsigned int ReadPressure = 64;
  static const unsigned long int PressureError = 0x7fffffff;
  static unsigned long int pressure_min, pressure_max;
  static unsigned long int pressure_mid;
  static unsigned long int pressure_sum;
  static float a, b;
  static int smooth;
  static QArray<uint32> last_press;

  static void setCoefficients(void);

public:
  TSPressure(void);
  
  typedef struct {int oo, op, po, om, mo;} press_dist;

  static int pressure(void);
  static void clear(void);
  static unsigned long int rarePressure(void);
  static unsigned long int pressureMin(void);
  static unsigned long int pressureMid(void);
  static unsigned long int pressureMax(void);
  static void setPressureMin(unsigned long int i);
  static void setPressureMid(unsigned long int i);
  static void setPressureMax(unsigned long int i);
  static void setPressureMin(press_dist);
  static void setPressureMax(press_dist);
  static int getSmooth(void);
  static void setSmooth(int);
  static void setWidth(int w) {width = w;}
  static void setHeight(int h) {height = h;}

 private:
  static press_dist p_max, p_min;
  static uint width, height;

  static int min_max_press(int, int, uint, uint, int, int, int);
  static int min_max_press(int x, int y, const press_dist&);
  static int pressure(long int);

 public:
  static int min_press(int x, int y);
  static int max_press(int x, int y);
  static int pressure(int x, int y);

};

class PressureSensor : public QWidget {
  Q_OBJECT
  
  unsigned long int pressure;

public:
  PressureSensor(QWidget*);
  
  void mouseMoveEvent(QMouseEvent *);

signals:
  void pressed(unsigned long int);
};

class QGrid;
class QVBox;
class QHBox;
class QLabel;
class QLineEdit;
class QCheckBox;
class QPushButton;

class TSDialog : public QDialog {
  Q_OBJECT

private:
  QVBox *main_box;
  PressureSensor *canvas;
  QHBox *labels_box, *nums_box, *smooth_box;
  QGrid *supple_grid;
  QLabel *l_min, *l_mid, *l_max, *l_smooth;
  QLineEdit *e_min, *e_mid, *e_max, *e_smooth;
  QPushButton *east_btn, *west_btn, *south_btn, *north_btn;
  QLineEdit *east_min, *west_min, *south_min, *north_min;
  QLineEdit *east_max, *west_max, *south_max, *north_max;

 public:
  enum NEWS {NEWSNone = 0, NEWSEast, NEWSWest,
	     NEWSNorth, NEWSSouth, NumOfNEWS};

 private:
  NEWS news;

  static unsigned long int min_pressure, max_pressure;
  static unsigned long int mid_pressure;

  void show_values(void);
  
public:
  TSDialog(void);
  ~TSDialog(void);

  void showValues(void) {show_values();}
  unsigned long int minPressure(void) {return min_pressure;}
  unsigned long int midPressure(void) {return mid_pressure;}
  unsigned long int maxPressure(void) {return max_pressure;}
  void setMinPressure(unsigned long int p) {min_pressure = p;}
  void setMidPressure(unsigned long int p) {mid_pressure = p;}
  void setMaxPressure(unsigned long int p) {max_pressure = p;}
  unsigned long int minPressure(NEWS);
  unsigned long int maxPressure(NEWS);
  void setMinPressure(NEWS, unsigned long int);
  void setMaxPressure(NEWS, unsigned long int);
  int smooth(void);
  void setSmooth(int);

 public:
  void mousePressEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *e) {mousePressEvent(e);}
  void mouseReleaseEvent(QMouseEvent *);

public slots:
  void pressureChanged(unsigned long int);
  void min_pressure_changed(const QString &);
  void mid_pressure_changed(const QString &);
  void max_pressure_changed(const QString &);
  void button_pressed(int);
  void newsPressureChanged(unsigned long int);
  void start_east(void) {news = NEWSEast; grabMouse();}
  void start_west(void) {news = NEWSWest; grabMouse();}
  void start_south(void) {news = NEWSSouth; grabMouse();}
  void start_north(void) {news = NEWSNorth; grabMouse();}
};

#endif		//  TSPRESSURE_H
