/*
  z_color_adjust.cpp
  by Saki Sakira <sakira@sun.dhis.portside.net>
  from 2005 May 28
*/

#include "qstring.h"
#include "qfile.h"
#include "qdir.h"
#include "qfileinfo.h"
#include "z_color_adjust.h"

// ZColorAdjust

const QString ZColorAdjust::ColorAdjustFileName("Settings/ZColorAdjust.conf");

ZColorAdjust::ZColorAdjust(void) {
  _adjust_table = new uint16[0x10000];
}

ZColorAdjust::~ZColorAdjust(void) {
  delete _adjust_table;
}

void ZColorAdjust::initAdjustTable(void) {
  for (uint16 i = 1; i != 0; i ++)
    _adjust_table[i] = i;
}

void ZColorAdjust::loadAdjustTable(void) {
  QString path = QDir::homeDirPath();

  if (path.right(1) != "/")
    path += "/";

  path += ColorAdjustFileName;
  QFile adjust_f(path);

  if (!adjust_f.exists()) {
    initAdjustTable();
    return;
  }

  int size = adjust_f.size();
  unsigned char str[size + 1];

  str[size] = 0;
  adjust_f.open(IO_ReadOnly);
  adjust_f.flush();
  size = adjust_f.readBlock((char*)str, size);
  adjust_f.close();

  if (size < 256 * 3) {
    initAdjustTable();
    return;
  }

  uint r, g, b;
  for (uint16 i = 1; i != 0; i ++) {
    r = (i >> 11) << 3;
    g = ((i >> 5) << 2) & 255;
    b = (i << 3) & 255;
    r = str[0 * 256 + r];
    g = str[1 * 256 + g];
    b = str[2 * 256 + b];
    r = (r >> 3) << 11;
    g = (g >> 2) << 5;
    b = (b >> 3);
    _adjust_table[i] = r | g | b;
  }
}

uint16* ZColorAdjust::adjust_table(bool use_adjust) {
  if (use_adjust)
    loadAdjustTable();
  else
    initAdjustTable();

  return _adjust_table;
}
